/**
 * Ein symbolisches Auto.
 * 
 * @author Bernd Gramlich
 * @version 2012-11-29
 */
public class Auto implements Beweglich
{
    // Bestandteile als Referenzen auf Graphikobjekte.
    private Vieleck karosserie;
    private Ellipse vorderrad;
    private Ellipse hinterrad;
    
    /**
     * Konstruktor.
     * Erzeuge ein Auto der Strichstrke a
     * an der Position (x|y) mit den gewnschten Farben.
     */
    public Auto (int a, int x, int y,
                  String karosseriefarbe, String reifenfarbe)
    {
        karosserie = new Vieleck ();
        vorderrad  = new Ellipse ();
        hinterrad  = new Ellipse ();
        
        karosserie.ergaenzeEcke (x       , y +  5*a);
        karosserie.ergaenzeEcke (x       , y + 15*a);
        karosserie.ergaenzeEcke (x + 60*a, y + 15*a);
        karosserie.ergaenzeEcke (x + 60*a, y +  5*a);
        karosserie.ergaenzeEcke (x + 45*a, y +  5*a);
        karosserie.ergaenzeEcke (x + 35*a, y       );
        karosserie.ergaenzeEcke (x + 15*a, y       );
        karosserie.ergaenzeEcke (x +  5*a, y +  5*a);
        karosserie.setzeFarbe (karosseriefarbe);
        
        hinterrad.setzeBreite (10*a);
        hinterrad.setzeHoehe  (10*a);
        hinterrad.setzePosition (x +  5*a, y +  8*a);
        hinterrad.setzeFarbe (reifenfarbe);
        
        vorderrad.setzeBreite (10*a);
        vorderrad.setzeHoehe  (10*a);
        vorderrad.setzePosition (x + 45*a, y +  8*a);
        vorderrad.setzeFarbe (reifenfarbe);
        
        macheSichtbar ();
    }
    
    /**
     * Mache das Auto sichtbar.
     */
    public void macheSichtbar ()
    {
        karosserie.macheSichtbar ();        
        vorderrad.macheSichtbar ();
        hinterrad.macheSichtbar ();
    }
    
    /**
     * Mache das Auto unsichtbar.
     */
    public void macheUnsichtbar ()
    {
        karosserie.macheUnsichtbar ();        
        vorderrad.macheUnsichtbar ();
        hinterrad.macheUnsichtbar ();
    }
    
    /**
     * Verschiebe das Auto um den Vektor (dx|dy).
     */
    public void verschiebe (int dx, int dy)
    {
        macheUnsichtbar ();
        
        karosserie.verschiebe (dx, dy);        
        vorderrad.verschiebeWaagrecht (dx);
        vorderrad.verschiebeSenkrecht (dy);
        hinterrad.verschiebeWaagrecht (dx);
        hinterrad.verschiebeSenkrecht (dy);
        
        macheSichtbar ();
    }
    
    /**
     * Nenne die Karosseriefarbe.
     */
    public String gibKarosseriefarbe ()
    {
        return karosserie.gibFarbe ();
    }
    
    /**
     * Nenne die Reifenfarbe.
     */
    public String gibReifenfarbe ()
    {
        return vorderrad.gibFarbe ();
    }
}
