/**
 * Ein symbolischer Baum.
 * 
 * @author Bernd Gramlich
 * @version 2012-11-29
 */
public class Baum
{
    // Bestandteile als Referenzen auf Graphikobjekte.
    private Rechteck stamm;
    private Ellipse krone;
    
    // Strichstrke.
    private int staerke;
    
    // Koordinaten der linken oberen Ecke.
    private int positionX;
    private int positionY;
    
    // Alter des Baumes in Jahren.
    private int alter;
    
    /**
     * Konstruktor.
     * Erzeuge einen Baum der Strichstrke a
     * an der Position (x|y), der j Jahre alt
     * ist.
     */
    public Baum (int a, int x, int y, int j)
    {
        stamm = new Rechteck ();
        krone = new Ellipse ();
        
        staerke   = a;
        positionX = x;
        positionY = y;
        
        setzeAlter (j);
        
        stamm.setzeFarbe ("braun");
        krone.setzeFarbe ("grn");
        
        passeGroesseAn ();
        
        macheSichtbar ();
    }
    
    /**
     * Passe die Gre des Baums an sein Alter an.
     */
    private void passeGroesseAn ()
    {
        int faktor = (int) Math.floor (Math.sqrt (alter/100.0) * 100);
        
        stamm.setzePosition (positionX + 40*staerke - faktor*staerke/20, positionY + (1000-9*faktor)*staerke/20);
        stamm.setzeBreite (faktor*staerke/10);
        stamm.setzeHoehe (9*faktor*staerke/20);
        
        krone.setzePosition (positionX + 40*staerke - faktor*staerke/7, positionY + (100-faktor)*staerke/2);
        krone.setzeBreite (2*faktor*staerke/7);
        krone.setzeHoehe (faktor*staerke/3);
    }
    
    /**
     * Mache den Baum sichtbar.
     */
    public void macheSichtbar ()
    {
        stamm.macheSichtbar ();        
        krone.macheSichtbar ();
    }
    
    /**
     * Mache den Baum unsichtbar.
     */
    public void macheUnsichtbar ()
    {
        stamm.macheUnsichtbar ();        
        krone.macheUnsichtbar ();
    }
    
    /**
     * Setze das Alter des Baumes.
     * Das Alter soll zwischen einem und hundert
     * Jahren betragen.
     */
    public void setzeAlter (int neuesAlter)
    {
        alter = neuesAlter;
        
        if (alter > 100)
        {
            alter = 100;
        }
        else if (alter < 1)
        {
            alter = 1;
        }
        
        passeGroesseAn ();
    }
    
    /**
     * Nenne das Alter des Baumes.
     */
    public int gibAlter ()
    {
        return alter;
    }
}
