/**
 * Piktogramm einer Person als abstrakte Klasse.
 * 
 * @author Bernd Gramlich
 * @version 2013-11-04
 */
public abstract class Person implements Beweglich
{
    // Bestandteile als Referenzen auf Graphikobjekte.
    protected Kreissektor haupthaar;
    protected Vieleck langhaar;
    protected Ellipse kopf;
    protected Vieleck kragen;
    protected Vieleck jacket;
    protected Vieleck rock;
    protected Vieleck hand1;
    protected Vieleck hand2;
    protected Vieleck hose;
    protected Vieleck schuh1;
    protected Vieleck schuh2;
    protected Beschriftung namensschild;
    protected String personenname;
    
    /**
     * Konstruktor.
     * Erzeuge ein Piktogramm der Strichstrke a
     * an der Position (x|y) mit den gewnschten Farben.
     */
    public Person (int a, int x, int y, String name,
                 String hautfarbe, String haarfarbe, String schuhfarbe,
                 String kleiderfarbe1, String kleiderfarbe2, String kleiderfarbe3)
    {
        haupthaar    = new Kreissektor ();
        langhaar     = new Vieleck ();
        kopf         = new Ellipse ();
        kragen       = new Vieleck ();
        jacket       = new Vieleck ();
        rock         = new Vieleck ();
        hand1        = new Vieleck ();
        hand2        = new Vieleck ();
        hose         = new Vieleck ();
        schuh1       = new Vieleck ();
        schuh2       = new Vieleck ();
        namensschild = new Beschriftung ();
        
        haupthaar.setzePosition (x + 12*a, y);
        haupthaar.setzeRadius (8*a);
        haupthaar.setzeAnfangswinkel (0);
        haupthaar.setzeGesamtwinkel (180);
        haupthaar.setzeFarbe (haarfarbe);
        
        langhaar.setzeFarbe (haarfarbe);
        
        kopf.setzePosition (x + 14*a, y +   2*a);
        kopf.setzeBreite (12*a);
        kopf.setzeHoehe (16*a);
        kopf.setzeFarbe (hautfarbe);
        
        hand1.setzeFarbe (hautfarbe);
        hand2.setzeFarbe (hautfarbe);
        
        jacket.setzeFarbe (kleiderfarbe1);
        
        rock.setzeFarbe (kleiderfarbe2);
        
        hose.setzeFarbe (kleiderfarbe3);
        
        schuh1.setzeFarbe (schuhfarbe);
        schuh2.setzeFarbe (schuhfarbe);
        
        personenname = name;
        namensschild.setzePosition (x + 20*a, y + 85*a);
        namensschild.setzeSchriftgroesse (5*a);
        namensschild.setzeFarbe (kleiderfarbe1);
        namensschild.setzeInhalt (personenname);
        namensschild.zentriere ();
    }
    
    /**
     * Mache das Piktogramm sichtbar.
     */
    public void macheSichtbar ()
    {
        haupthaar.macheSichtbar ();        
        langhaar.macheSichtbar ();
        jacket.macheSichtbar ();
        hand1.macheSichtbar ();
        hand2.macheSichtbar ();
        kragen.macheSichtbar ();
        hose.macheSichtbar ();
        rock.macheSichtbar ();
        kopf.macheSichtbar ();
        schuh1.macheSichtbar ();
        schuh2.macheSichtbar ();
        namensschild.macheSichtbar ();
    }
    
    /**
     * Mache das Piktogramm sichtbar.
     */
    public void macheUnsichtbar ()
    {
        haupthaar.macheUnsichtbar ();        
        langhaar.macheUnsichtbar ();
        jacket.macheUnsichtbar ();
        hand1.macheUnsichtbar ();
        hand2.macheUnsichtbar ();
        kragen.macheUnsichtbar ();
        rock.macheUnsichtbar ();
        kopf.macheUnsichtbar ();
        hose.macheUnsichtbar ();
        schuh1.macheUnsichtbar ();
        schuh2.macheUnsichtbar ();
        namensschild.macheUnsichtbar ();
    }
    
    /**
     * ndere den Namen des Piktogramms.
     */
    public void setzeName (String neuerName)
    {
        personenname = neuerName;
        namensschild.setzeInhalt (personenname);
    }
    
    /**
     * Nenne den Namen des Piktogramms.
     */
    public String gibName ()
    {
        return personenname;
    }
    
    /**
     * Verschiebe die Person um den Vektor (dx|dy).
     */
    public void verschiebe (int dx, int dy)
    {
        macheUnsichtbar ();
        
        haupthaar.verschiebeWaagrecht (dx);
        haupthaar.verschiebeSenkrecht (dy);
        langhaar.verschiebe (dx, dy);
        jacket.verschiebe (dx, dy);
        hand1.verschiebe (dx, dy);
        hand2.verschiebe (dx, dy);
        kragen.verschiebe (dx, dy);
        rock.verschiebe (dx, dy);
        kopf.verschiebeWaagrecht (dx);
        kopf.verschiebeSenkrecht (dy);
        hose.verschiebe (dx, dy);
        schuh1.verschiebe (dx, dy);
        schuh2.verschiebe (dx, dy);
        namensschild.verschiebeWaagrecht (dx);
        namensschild.verschiebeSenkrecht (dy);
        
        macheSichtbar ();
    }
    
    /**
     * Gib an, ob dieses Piktogramm weiblich ist.
     */
    public abstract boolean istWeiblich ();
    
    /**
     * Nenne die Haarfarbe der Person.
     */
    public String gibHaarfarbe ()
    {
        return haupthaar.gibFarbe ();
    }
    
    /**
     * Nenne die erste Kleiderfarbe der Person.
     */
    public String gibKleiderfarbe1 ()
    {
        return jacket.gibFarbe ();
    }
    
    /**
     * Nenne die zweite Kleiderfarbe der Person.
     */
    public String gibKleiderfarbe2 ()
    {
        return rock.gibFarbe ();
    }
    
    /**
     * Nenne die dritte Kleiderfarbe der Person.
     */
    public String gibKleiderfarbe3 ()
    {
        return hose.gibFarbe ();
    }
}
